#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Утилита для разбора XML с единым реестром Белоруссии
Реестр преобразуется в простой текстовый файл, подходящий для Carbon Reductor
"""

__author__ = 'oleg'

import xml.etree.ElementTree as etree
import sys
from urlparse import urlparse
from dump_parser import filter_by_scheme, pick_port, process_http, process_rest
from dump_parser import file_to_list, print_list, SCHEMES_REST

HTTP_LIST, HTTPS_LIST, HTTP_PORTS, IP_BLOCK = [], [], [], []
IP_HTTPS_LIST, IP_FTP_LIST, OTHER_URLS, FULLWHITELIST = [], [], [], []


def strip_empty(items, empty=('-',)):
    """ В белорусском реестре вместо отсутствия значения <item>-</item> """
    return [item for item in items if item not in empty]


def process_elem(elem):
    """ Обрабатывает одну запись в реестре. Распихивает её данные по глобальным
    спискам, предварительно сформировав свои """

    domain_list = [unicode(d.text).strip() for d in elem.findall('dns')]
    ip_list = [ip.text for ip in elem.findall('ip') + elem.findall('ipSubnet')]
    urls = [unicode(url.text).strip() for url in elem.findall('url')]

    urls = strip_empty(urls)
    domain_list = strip_empty(domain_list)
    ip_list = strip_empty(ip_list)

    urls_http = filter_by_scheme(urls, 'http')
    urls_https = filter_by_scheme(urls, 'https')
    urls_ftp = filter_by_scheme(urls, 'ftp')
    urls_rest = [u for u in urls if urlparse(u).scheme not in SCHEMES_REST]

    port_list_http = [port for port in map(pick_port, urls_http) if port]
    port_list_rest = [port for port in map(pick_port, urls_rest) if port]
    urls_http = process_http(urls_http)
    process_rest(urls_rest, urls_http, ip_list, port_list_rest)
    IP_HTTPS_LIST.extend(ip_list)
    HTTP_PORTS.extend(port_list_http)

    # пытаемся избежать полной блокировки ip создав urls_http из других источников
    if urls_https:
        HTTPS_LIST.extend(domain_list)
        urls_http.extend([url.replace('https', 'http', 1)
                          for url in urls_https])
        urls_http = process_http(urls_http)

    if not urls_http and domain_list:
        urls_http = ["http://" + d for d in domain_list]

    if urls_ftp:
        IP_FTP_LIST.extend(ip_list)

    if not urls_http:
        IP_BLOCK.extend(ip_list)

    HTTP_LIST.extend(urls_http)


def print_lists():
    """ Печатает все списки на экран """
    lists = (
        (HTTP_LIST, "urls to load", "add_url"),
        (HTTPS_LIST, "https domains to resolv", "need_resolv"),
        (IP_BLOCK, "ip to full block", "add_ip_block"),
        (IP_HTTPS_LIST, "ip to block on https", "add_ip_https"),
        (HTTP_PORTS, "http ports", "add_http_port"),
        (IP_FTP_LIST, "ftp ip", "add_ip_ftp"),
        (OTHER_URLS, "other urls by ip + port", "add_ip_port"),
    )
    for list_structure in lists:
        print_list(list_structure)


def process_dump_xml(dump):
    """ Обрабатывает содержимое dump.xml """
    for entry in etree.parse(dump).getroot():
        process_elem(entry)

if __name__ == '__main__':
    FULLWHITELIST = file_to_list('/usr/local/Reductor/lists/our.fullwhitelist')
    # DUMP = "/usr/local/Reductor/reductor_container/gost-ssl/php/dump.xml"
    DUMP = "reductor_container/gost-ssl/php/dump_by.xml"
    if len(sys.argv) == 2:
        DUMP = sys.argv[1]
    process_dump_xml(DUMP)
    print_lists()
