# coding: utf-8
""" Тесты обработки единого реестра """

import os
from unittest import TestCase
from dump_parser import process_dump_xml
from dump_parser import HTTP_LIST, HTTP_PORTS, HTTPS_PORTS, IP_HTTPS_LIST
from dump_parser import IP_FTP_LIST, IP_BLOCK, HTTPS_LIST, OTHER_URLS


def process(filename):
    """ 'симлинк' для удобства """
    test_data_dir = '/Users/oleg/git/reductor_userspace/test_data/'
    process_dump_xml(test_data_dir + filename)


class CarbonTestCase(TestCase):
    """ Чтобы по умолчанию дебаг был включен на полную """

    def setup(self):
        """ включаем дебаг на полную """
        # pylint: disable=C0103
        self.longMessage = True
        # pylint: enable=C0103


class DumpParserTests(CarbonTestCase):
    """ Тесты разбора различных entry в реестре """

    @staticmethod
    def test_http_port():
        """ проверка того что список http портов заполняется """
        process('http_port.xml')
        assert u'http://example4.com:24680/request' in HTTP_LIST
        assert u'24680' in HTTP_PORTS

        os.environ['BLOCK_HTTPS_IP_EVEN_IF_HTTP'] = "0"
        process('http_port.xml')
        assert u'http://example4.com:24680/request' in HTTP_LIST
        assert '1.2.3.14' not in IP_HTTPS_LIST

        os.environ['BLOCK_HTTPS_IP_EVEN_IF_HTTP'] = "1"
        process('http_port.xml')
        assert u'http://example4.com:24680/request' in HTTP_LIST
        assert '1.2.3.14' in IP_HTTPS_LIST

        del os.environ['BLOCK_HTTPS_IP_EVEN_IF_HTTP']

    @staticmethod
    def test_https_port():
        """ тестируем совсем уже странности """
        process('https_ip_on_nonstandard_port_as_url.xml')
        assert u'16869' in HTTPS_PORTS
        assert '52.29.64.231 16869' in OTHER_URLS

    @staticmethod
    def test_https_default_port():
        """ тестируем то, что мы отрезаем 443 порт у https URL """
        process('https_with_443_port.xml')
        assert u'443' not in HTTPS_PORTS
        assert 'new6.betaland.com:443' not in HTTPS_LIST
        assert 'new6.betaland.com' in HTTPS_LIST
        assert 'http://new6.betaland.com:443/Sport/Default.aspx' not in HTTP_LIST
        assert 'http://new6.betaland.com/Sport/Default.aspx' in HTTP_LIST

    @staticmethod
    def test_ftp():
        """ проверка заполнения ipset для ftp """
        process('ftp.xml')
        assert IP_FTP_LIST == ['1.2.3.4']

    @staticmethod
    def test_http_normal():
        """ проверка того что обычные http не приводят к полному блоку """
        process('http_normal.xml')
        assert 'http://example.com/request' in HTTP_LIST
        assert '1.2.3.5' not in IP_HTTPS_LIST
        assert '1.2.3.5' not in IP_BLOCK

        os.environ['BLOCK_HTTPS_IP_EVEN_IF_HTTP'] = "0"
        process('http_normal.xml')
        assert 'http://example.com/request' in HTTP_LIST
        assert '1.2.3.5' not in IP_HTTPS_LIST
        assert '1.2.3.5' not in IP_BLOCK

        os.environ['BLOCK_HTTPS_IP_EVEN_IF_HTTP'] = "1"
        process('http_normal.xml')
        assert 'http://example.com/request' in HTTP_LIST
        assert '1.2.3.5' in IP_HTTPS_LIST
        assert '1.2.3.5' not in IP_BLOCK
        del os.environ['BLOCK_HTTPS_IP_EVEN_IF_HTTP']

    @staticmethod
    def test_https():
        """ проверка разбора https, entry = resovl + ip + http url """
        process('https.xml')
        assert len(HTTPS_LIST) > 0
        assert 'example.com' in HTTPS_LIST
        assert '1.2.3.6' in IP_HTTPS_LIST
        assert 'http://example.com/request_https' in HTTP_LIST

    @staticmethod
    def test_rest_ip_port():
        """ newcamd и подобные не должны попадать в HTTP_LIST """
        process('rest_ip_port.xml')
        assert '1.2.3.8 123' in OTHER_URLS
        assert 'Newcamd://example.com:123/' not in HTTP_LIST
        assert 'newcamd://example.com:123/' not in HTTP_LIST

    @staticmethod
    def test_multiple_url():
        """ все URL записи должны попадать в HTTP LIST """
        process('multiple_url.xml')
        assert 'http://example.com/request_3' in HTTP_LIST
        assert 'http://example.com/request_4' in HTTP_LIST

    @staticmethod
    def test_multiple_http_with_domain():
        """ но если один URL - домен - должен попасть только он """
        process('multiple_url_domain.xml')
        assert 'http://example1.com/request_1' not in HTTP_LIST
        assert 'http://example1.com/request_2' not in HTTP_LIST
        assert 'http://example1.com' in HTTP_LIST

    @staticmethod
    def test_multiple_https_with_domain():
        """ в случае если URL - https, поведение не должно отличаться """
        process('multiple_url_domain_real.xml')
        assert 'http://example2.com/products/#dc1' not in HTTP_LIST
        assert 'http://example2.com/products/' not in HTTP_LIST
        assert 'http://example2.com' in HTTP_LIST

    @staticmethod
    def test_domain_and_ip():
        """ при отсутствии URL, заменяем его доменом """
        process('domain_ip.xml')
        assert 'http://example3.com' in HTTP_LIST
        assert '1.2.3.11' in IP_HTTPS_LIST
        assert '1.2.3.12' in IP_HTTPS_LIST

    @staticmethod
    def test_ip_only():
        """ Если нет ни URL ни домена - баним целиком """
        process('ip_only.xml')
        assert '1.2.3.7' in IP_BLOCK
