#!/bin/bash

. /usr/local/Reductor/etc/const
. $CONFIG

# please redefine them in hook:
# http://docs.carbonsoft.ru/pages/viewpage.action?pageId=55934984

CUSTOM_EXIT=1
LOCKDIR=/var/lock/reductor/
LOCKFILE=$LOCKDIR/reload_ip.sh.lock
LOCKFILE2=$LOCKDIR/prepare_lists.sh.lock

trap __exit EXIT

__custom_exit() {
        flock -u 3
        flock -u 4
}

ip_block_full_add() { return 0; }
ip_block_full_del() { return 0; }
ip_block_full_get() { return 0; }

prepare() {
	mkdir -p $TMPDIR $LOCKDIR
	exec 3>$LOCKFILE
	exec 4>$LOCKFILE2
	echo "- Проверяем lock ip"
	flock -x 3
	echo "- Проверяем lock url/ip"
	flock -x 4
}

use_hook() {
	hook=$HOOKDIR/${0##*/}
	[ -x $hook ] && . $hook
}

diff_data_only() {
	egrep '^(\+|\-)' | egrep -v '^(\+|\-){3}'
}

diff_to_events() {
	sed -e 's/-/ip_block_full_add /; s/+/ip_block_full_del /'
}

files_to_events() {
	local old="$1"
	local new="$2"
	sort_lists $old $new
	diff -U 0 "$new" "$old" | diff_data_only | diff_to_events
}

sort_lists() {
	for list in "$@"; do
		touch $list
		LANG='' sort -u $list > $list.sorted
		mv -f $list.sorted $list
	done
}

process_events() {
	while read event ip _; do
		log $event $ip
		$event $ip
	done
}

main() {
	log "tick begin"
	local DATA_DIR=/var/lib/reductor/
	local FILE=$DATA_DIR/ip_block_full
	local NEW=$FILE.new
	local OLD=$FILE.old
	local IP_BLOCK_FULL_LIST=$LISTDIR/ip_block.load
	mkdir -p $DATA_DIR/
	prepare
	ip_block_full_get > $OLD
	cp $IP_BLOCK_FULL_LIST $NEW
	files_to_events $OLD $NEW | process_events
	log "tick end"
}

use_hook
main "$@"
exit 0
