#!/bin/bash

. /usr/local/Reductor/etc/const
. $CONFIG

set -eu
trap __exit EXIT

file2ipset_safe() {
	local ipset=$1
	local file=$2
	ipset flush $ipset
	while read item _; do
		ipset add $ipset $item || true
	done < $file
}

file2ipset_fast() {
	local ipset=$1
	local file=$2
	ipset save $ipset | grep ^create | sed 's/create /ipset &_/g' | bash
	ipset flush _$ipset
	grep -v ^$ $file | sed -e "s/^/add _$ipset /" | sort -u | ipset restore
	ipset swap _$ipset $ipset
	ipset flush _$ipset
	ipset destroy _$ipset
}

file2ipset() {
	if [ ! -s "$2" ]; then
		echo "- Пропускаем загрузку, т.к. $2 пустой или отсутствует"
		return 0
	fi
	file2ipset_fast "$@" || file2ipset_safe "$@"
}

load_ip() {
	if [ "${filter['https_by_ip']}" = '1' ]; then
		file2ipset https_reductor $LISTDIR/ip.load
		file2ipset http_reductor $LISTDIR/ip_block.load
		if [ "${filter['ipv6']:-0}" = '1' ]; then
			file2ipset https_reductor_ipv6 $LISTDIR/ip6.load
		fi
	fi

	for proto in http https; do
		if [ -s $LISTDIR/our.white_${proto}_iplist ]; then
			file2ipset white_${proto}_reductor $LISTDIR/our.white_${proto}_iplist
		fi
	done

	file2ipset ftp_reductor $LISTDIR/ip_ftp.load
}

load_ports() {
	file2ipset http_ports $LISTDIR/http.ports
	ipset add http_ports 80 || true
}

load_ip_ports() {
	if [ -f $LISTDIR/rkn.ipport ]; then
		awk '{print $1}' $LISTDIR/rkn.ipport > $TMPDIR/rkn.ipport_ip.$$
		awk '{print $2}' $LISTDIR/rkn.ipport > $TMPDIR/rkn.ipport_port.$$
		file2ipset ip_port_ban $TMPDIR/rkn.ipport_ip.$$
		file2ipset tcp_ports $TMPDIR/rkn.ipport_port.$$
		file2ipset udp_ports $TMPDIR/rkn.ipport_port.$$
		rm -f $TMPDIR/rkn.ipport.$$ $TMPDIR/rkn.ipport_ip.$$ $TMPDIR/rkn.ipport_port.$$
	fi
}

use_hook() {
	hook="$HOOKDIR/${0##*/}"
	if [ -x "$hook" ]; then
		. "$hook"
	fi
}

client_prepare_hook() {
	return 0
}

main() {
	client_prepare_hook
	load_ip
	load_ports
	load_ip_ports
}

use_hook
main "$@"
