#!/bin/bash

. /usr/local/Reductor/etc/const
. $CONFIG
. $MENULIB

if [ "$network_work_ok" = '1' ]; then
	exit 255
fi

baseopts="--stdout --output-fd 2 --colors --trim --backtitle '${app[name]}' --no-shadow --item-help"
baseopts_eth="$baseopts --cancel-label 'Далее' --ok-label 'Добавить'"
baseopts_vlan="$baseopts --cancel-label 'Завершить' --ok-label 'Добавить'"
vlan_opts="--no-label Добавить_vlan --yes-label Пропустить --yesno"

bridge_if=br0

ASK_MIRROR_SETUP="Настроить зеркало трафика автоматически? Предыдущие настройки будут сохранены в каталоге $USERDIR/backup"
ASK_VLAN_SETUP="Если на ${app['name']} приходит тэгированый (vlan) трафик, то лучше снимать тэгирование на зеркальном порту коммутатора. Если такой возможности нет, можно настроить vlan на этом сервере. Настроить vlan сейчас?"

ask_custom() {
	yes="$1"
	no="$2"
	shift 2
	dialog --yes-label "$yes" --no-label "$no" --yesno "$@" 10 50
}

ask_about_setup() {
	ask_custom "Далее" "Отмена" "$ASK_MIRROR_SETUP"
}

get_defroute_dev() {
	ip r | grep -m1 default | egrep -o [a-z]+[0-9]+
}

get_iface_mac_list() {
        tmpfile=/tmp/iface_mac.list
	defroute_dev=$(get_defroute_dev)
        ip -o l | egrep ": (eth|em)" | grep -v "@" | grep -vw "${defroute_dev// /}" > $tmpfile
        while read _ iface _ _ _ _ _ _ _ _ _ _ mac _; do
                echo -n "${iface%:} $mac '' "
        done < $tmpfile
        echo
        rm -f $tmpfile
}

show_interface_checkbox_group() {
	return 0
}

get_bridge_dev() {
	cat /tmp/bridge_iface
}

remove_dev_from_all_bridges() {
	for bridge in $(egrep -o br[0-9]+ /proc/net/dev); do
		brctl delif $bridge $1 &>/dev/null
	done
}

setup_mirror_device() {
	local device=$1

	if [ "$mirror_type" != 'l3' ]; then
		bridge="$(get_bridge_dev)"
	fi

	if ask_custom "Далее" "Отмена" "Добавить и настроить сканирование трафика с $device?"; then
		if [ "$mirror_type" == 'l2_eth' ]; then
			__create_eth_device_gen	"$device" $bridge && show_msg "$device настроен на приём трафика с зеркала"
		fi

		if [ "$mirror_type" == 'l2_vlan' ]; then
			__create_eth_device_l2_vlan_gen	"$device" $bridge && show_msg "$device настроен на приём трафика с зеркала"
		fi

		if [ "$mirror_type" == 'l3' ]; then
			local ip
			ip="$(dialog_read "Введите IP/маску для $device, например 192.168.100.15/24" 10 80)" || return 0
			local netmask
			if [ "$ip" == "${ip////}" ]; then
				netmask=$(dialog_read "Введите маску для $device:") || return 0
				netmask="$(ipcalc --netmask $ip/$netmask)"
				netmask="${netmask##*=}"
			else
				netmask="$(ipcalc --netmask $ip)"
				netmask="${netmask##*=}"
				ip="${ip%%/*}"
			fi
			__create_eth_device_l3_gen "$device" "$ip" "$netmask" && show_msg "$device настроен на приём трафика с зеркала"
			remove_dev_from_all_bridges "$device"
		fi
	fi
}

get_mirror_device() {
	get_iface_mac_list
}

select_mirror_device() {
	mirror_type="$1"
	ifaces=$(get_iface_mac_list)
	echo "ifaces = $ifaces"
	while true; do
		local __baseopts="$baseopts_eth"
		if [ "$mirror_type" == 'l2_eth' ]; then
			local __baseopts="$baseopts_vlan"
		fi
		iface=$(eval "dialog $__baseopts --menu 'Добавить интерфейсы для сканирования трафика:' 11 70 11 $ifaces 'Далее' '' '' 3>&1 1>&2 2>&3")
		[ "$?" != '0' ] && echo "Not null" && break
		[ "$iface" = 'Далее' ] && echo "Далее" && break
		setup_mirror_device "$iface" "$mirror_type"
	done
}

__create_vlan_device() {
	eth=$1
	bridge=$(get_bridge_dev)
	ask_custom "Создать" "Отмена" "Создать и настроить vlan интерфейс поверх $eth?" || return 0
	vlan_tag=$(dialog_read "Введите номер тэга vlan:") || return 0
	__create_vlan_device_gen $eth $vlan_tag $bridge && show_msg "Поверх $eth создан и настроен vlan $vlan_tag"
}

get_vlan_device() {
	get_mirror_device
	while true; do
		iface=$(eval "dialog $baseopts_vlan --menu 'Добавить vlan на интерфейсы...' 11 70 11 $ifaces 'Далее' '' '' 3>&1 1>&2 2>&3")
		[ "$?" != '0' ] && echo "Not null" && break
		[ "$iface" = 'Готово' ] && echo "Готово" && break
		__create_vlan_device $iface
	done

}

create_vlan() {
	echo "Создаю vlan интерфейсы"
	ifaces=$(get_iface_mac_list)
	get_vlan_device
}

bridge_exist() {
	bridge=$1
	ip link | fgrep -wqm1 "$bridge:" && return 0
	[ -f $NETWORK_SCRIPTS/ifcfg-$bridge ]
}

get_bridge_if() {
	br_num=0
	while true; do
		bridge_exist br$br_num || break
		((br_num++))
	done
	echo br$br_num | tee /tmp/bridge_iface
}

setup_bridge() {
	echo "Создаю bridge интерфейс"
	bridge_if=$(get_bridge_if)
	conf=$NETWORK_SCRIPTS/ifcfg-$bridge_if
	tmplt=$MAINDIR/contrib/tmplts/ifcfg-bridge
	ip=$(get_free_ip)
	if ask_custom "Далее" "Отмена" "Создать и настроить bridge $bridge_if?"; then
		genconfig conf=$conf tmplt=$tmplt bridge=$bridge_if ip=$ip
	fi
}

replace_ifup() {
	mkdir -p $BACKUPDIR
	cp -v $NETWORK_SCRIPTS/ifup-eth "$BACKUPDIR/ifup-eth.$(date +%s)"
	cp -v $CONTRIB/$NETWORK_SCRIPTS/ifup-eth $NETWORK_SCRIPTS/ifup-eth
}

# magik, tmplt & conf comes from param
genconfig() {
	for param in "$@"; do
		[[ "$param" = *=* ]] && eval $param
	done
	while read line; do
		eval echo $line
	done < $tmplt > $conf
}

ip_exist() {
	ip a | fgrep -wqm1 "$1" && return 0
	fgrep -wqm1 -r "$1" $NETWORK_SCRIPTS
}

get_free_ip() {
	while true; do
		ip=169.255.255.$((RANDOM%254 + 1))
		ip_exist $ip || break
	done
	echo $ip
}

__create_vlan_device_gen() {
	eth=$1
	tag=$2
	conf=$NETWORK_SCRIPTS/ifcfg-$eth.$tag
	tmplt=$MAINDIR/contrib/tmplts/ifcfg-vlan
	genconfig conf=$conf tmplt=$tmplt eth=$eth tag=$tag "ip=$(get_free_ip)" bridge=$3
}

__create_eth_device_gen() {
	eth=$1
	conf=$NETWORK_SCRIPTS/ifcfg-$eth
	tmplt=$MAINDIR/contrib/tmplts/ifcfg-eth
	genconfig conf=$conf tmplt=$tmplt eth=$eth ip=$(get_free_ip) bridge=$2
}

__create_eth_device_l2_vlan_gen() {
	eth=$1
	conf=$NETWORK_SCRIPTS/ifcfg-$eth
	tmplt=$MAINDIR/contrib/tmplts/ifcfg-eth-vlan
	genconfig conf=$conf tmplt=$tmplt eth=$eth ip=$(get_free_ip)
}

__create_eth_device_l3_gen() {
	eth=$1
	conf=$NETWORK_SCRIPTS/ifcfg-$eth
	tmplt=$MAINDIR/contrib/tmplts/ifcfg-eth-l3
	ip=$2
	netmask=$3
	genconfig conf=$conf tmplt=$tmplt eth=$eth ip=$ip netmask=$netmask
}

start_network_setup() {
	bkdir="$BACKUPDIR/network-scripts.$(date +%s)"
	mkdir -p "$bkdir"
	cp -varup $NETWORK_SCRIPTS/* "$bkdir"
}

remove_old_settings() {
	defroute_dev=$(get_defroute_dev)
	find $NETWORK_SCRIPTS/ ! -name "ifcfg-$defroute_dev" | egrep ifcfg-"(em|eth|br)[0-9]*" > /tmp/unused_configs
	while read file; do
		if [[ "$file" = *br* ]]; then
			ip link set dev ${file##*ifcfg-} down
			brctl delbr ${file##*ifcfg-}
		fi
		rm -f $file
	done < /tmp/unused_configs
}

l2_vlan() {
	replace_ifup || terminate "Не удалось заменить ifup-eth, $GOTO_SUPPORT"
	setup_bridge || terminate "Если вы фильтруете трафик с коммутатора - без bridge не обойтись. Настройте его вручную."
	select_mirror_device l2_vlan
	create_vlan
	misc['skipnetworkcheck']='0'
	save_and_exit
	return 0
}

l2_eth() {
	replace_ifup || terminate "Не удалось заменить ifup-eth, $GOTO_SUPPORT"
	setup_bridge || terminate "Если вы фильтруете трафик с коммутатора - без bridge не обойтись. Настройте его вручную."
	select_mirror_device l2_eth
	misc['skipnetworkcheck']='0'
	save_and_exit
	return 0
}

l3() {
	select_mirror_device l3
	misc['skipnetworkcheck']='1'
	save_and_exit
	return 0
}

choose_mirror_type() {
	local options="$baseopts --menu 'Выберите тип зеркала' 10 60 5"
	local variants="'l2_eth' 'L2 mirror без VLAN' 'Настраивается на коммутаторе' 'l2_vlan' 'L2 mirror с VLAN' 'Настраивается на коммутаторе' 'l3' 'L3 mirror на IP' 'Настраивается на маршрутизаторах, обычно с помощью xt_TEE'"
	eval dialog "$options" "$variants"
}

main() {
	$BINDIR/autogit.sh /etc/sysconfig/network-scripts/ || true
	ask_about_setup || terminate "$NETWORK_SETUP_LINK"
	choose_mirror_type 2>$TMPDIR/mirror_type
	echo >> $TMPDIR/mirror_type
	read mirror_type < $TMPDIR/mirror_type
	if ! [[ "$mirror_type" =~ (l2_vlan|l2_eth|l3) ]]; then
		terminate "Надо было выбрать тип зеркала трафика"
	fi
	start_network_setup
	remove_old_settings
	$mirror_type
	show_msg "Настройка сети завершена, Carbon Reductor готов к приёму и фильтрации трафика. Если у вас возникнут трудности - $GOTO_SUPPORT"
	ask "Применить новые настройки сети?" && /etc/init.d/network restart
	$BINDIR/autogit.sh /etc/sysconfig/network-scripts/ || true
}

${1:-main}
