#!/bin/bash

. /usr/local/Reductor/etc/const
. /etc/init.d/functions
. $CONFIG

mkdir -p $TMPDIR

remove_rules_by_pattern() {
	local table=$1 chain=$2 pattern=$3 iptables="${4:-iptables}"
	$iptables -t $table --line -nvL $chain | grep -i $pattern | tac > $TMPDIR/stop.sh
	while read num _; do
		$iptables -t $table -D $chain $num
	done < $TMPDIR/stop.sh
}

remove_reductor_rules() {
	remove_rules_by_pattern raw PREROUTING ip_port
	remove_rules_by_pattern filter FORWARD reductor
	remove_rules_by_pattern filter FORWARD dnsmatch
	remove_rules_by_pattern filter FORWARD DNSREPLY
	remove_rules_by_pattern filter FORWARD reductor ip6tables
	remove_rules_by_pattern filter INPUT reductor_admins
	remove_rules_by_pattern filter FORWARD http_ports
	remove_rules_by_pattern mangle PREROUTING http_ports
	remove_rules_by_pattern mangle PREROUTING ip_port
	remove_rules_by_pattern filter FORWARD ip_port
	remove_rules_by_pattern filter FORWARD custom_
}

remove_forbidden_rules() {
	remove_rules_by_pattern filter FORWARD FORBIDDEN
}

flush_and_destroy_ipset() { #auth, negbal, blocked чистятся здесь для простоты
	if ipset save | egrep ^add.*'(auth|negbal|blocked)' > $TMPDIR/billing_redirect_ipset.save.tmp; then
		mv $TMPDIR/billing_redirect_ipset.save{.tmp,}
	fi

	for list in auth negbal skip_check blocked reductor_admins http_ports {ftp,{white_,}http{s,},ip_port}_reductor https_reductor_ipv6; do
		if ipset l -o save | grep -q "create $list"; then
			ipset flush $list
			ipset destroy $list
		fi
	done

	# remove custom ipset if we have them
	for i in $(ipset save | grep '^create custom_' | awk '{print $2}'); do
		ipset flush $i
		ipset destroy $i
	done
}

umount_work() {
	chroot_umount
	for point in dev proc; do
		if mount | grep -q $CHROOTDIR/$point; then
			for dir in /usr/local/*/reductor_container/$point; do
				df -h "$dir" | egrep -q "^$point" && umount $dir
			done
		fi
	done
}

client_post_stop_hook() {
	return 0
}

use_hook() {
	hook=$HOOKDIR/${0##*/}
	[ -x $hook ] && . $hook
}

cleanup_global() {
	iptables -F
	iptables -t mangle -F
	iptables -t raw -F
	iptables -X
}

main() {
	echo "# Отключается Carbon Reductor:"
	echo "- Удаляются все фильтрующие правила..."
	cleanup_global
	remove_reductor_rules
	remove_forbidden_rules
	remove_rules_by_pattern filter FORWARD skip_check
	remove_rules_by_pattern filter FORWARD blocked
	remove_rules_by_pattern filter FORWARD negbal
	remove_rules_by_pattern filter FORWARD auth
	echo "- Чистка ipset созданных Carbon Reductor"
	flush_and_destroy_ipset
	umount_work
	echo "- Выгружаются модули ядра"
	module_unload xt_reductor
	module_unload ipt_FORBIDDEN
	module_unload ip6t_FORBIDDEN
	module_unload xt_dnsmatch
	module_unload ipt_DNSREPLY
	$BINDIR/billing_redirect.sh stop
	client_post_stop_hook
}

use_hook
main
