# coding:utf8
"""
Утилита для регистрации Carbon Reductor
"""
import sys
import os
import json
from carbon_reductor_reg import reg_file_save, reg_file_read, reg_file_send
from carbon_reductor_reg import REG_FILE_PATH, KEY_FILE_PATH, reg_number_from_file


def reg_file_update():
    """ Дописывает в файл новые данные введённые пользователем """
    if not all([os.environ.get(key) for key in ['company_name', 'email']]):
        print "No enough data to update registration info"
        exit(1)
    data = reg_file_read()
    for key in ['company_name', 'email', 'phone']:
        if os.environ.get(key):
            data['CRB-Reductor'][key] = os.environ[key]
    reg_file_save(data)


def reg_file_new():
    """ Добивается любым способом появления файла регномера """
    if not os.path.isfile(REG_FILE_PATH):
        reg_file_save()
    try:
        json.load(open(REG_FILE_PATH))
    except ValueError:
        os.remove(REG_FILE_PATH)
        reg_file_save()
    print reg_number_from_file()


def reg_data_check():
    """ Падает, если регфайл битый """
    data = reg_file_read()
    if 'CRB-Reductor' not in data:
        print "No enough data to complete registration (1)"
        exit(1)
    for key in ['company_name', 'email']:
        if key not in data['CRB-Reductor'] or data['CRB-Reductor'][key] == '':
            print key, "No enough data to complete registration (2)"
            exit(1)


def reg_data_print():
    """ Печатает информацию о регистрации и активации """
    reg_data_structure = {
        'reg_num': u'Регистрационный номер',
    }

    key_data_structure = {
        'cf_regactivate': u'Код активации',
        'cf_reghwid': u'Код установки',
    }

    try:
        data_set = (
            (reg_data_structure, reg_file_read().get('CRB-Reductor')),
            (key_data_structure, json.load(open(KEY_FILE_PATH)).get('CRB-Reductor')),
        )

        for desc, data in data_set:
            for key, desc in desc.items():
                print ("%s: %s" % (desc, data.get(key))).encode('utf-8')
    except IOError:
        print u'Нет активационных данных'.encode('utf-8')
    except ValueError:
        print u'Нет активационных данных'.encode('utf-8')


def usage():
    """ Выводит информацию о использовании """
    print """
    create - create new regnumber file
    update - write some known data to this file
    activate - send this file to carbonsoft's server, call registration and activation
    info - print info from regnumber file
    message - print message from activation server
    """
    exit(1)


def key_file_message():
    """ Возвращает сообщения с сервера активации """
    result_data = json.load(open(KEY_FILE_PATH))
    if not result_data.get('CRB-Reductor'):
        return
    message = result_data['CRB-Reductor'].get('msg')
    if message:
        print message.encode('utf-8')


def get_new_activate_key():
    reg_data_check()
    reg_file_send('reg')
    activate_info = reg_file_send('activate')
    if not activate_info:
        return
    with open(KEY_FILE_PATH, 'w') as keyfile:
        keyfile.write(activate_info)


def main():
    """ Вызывает нужное действие в зависимости от аргументов """
    if 'create' in sys.argv:
        reg_file_new()
    elif 'update' in sys.argv:
        reg_file_update()
    elif 'message' in sys.argv:
        key_file_message()
    elif 'info' in sys.argv:
        reg_data_print()
    elif 'activate' in sys.argv:
        get_new_activate_key()
    else:
        usage()


if __name__ == '__main__':
    main()
